# -*- coding: utf-8 -*-
"""
Created on 06.04.2019 by Ismail Baris
"""
from __future__ import division

import numpy as np

from rspy import Sensor
from rspy.auxiliary.types import __DTYPES__

n = np.random.randint(60, 101)
n2 = np.random.randint(120, 150)
n3 = np.random.randint(30, 50)

random_dtype = np.random.choice(__DTYPES__, 1)[0]
random_array = np.random.uniform(1, np.pi / 2, n2)
random_array2 = np.random.uniform(1, np.pi / 2, n3)

iza, vza, iaa, vaa, nbar = np.random.uniform(1, np.pi, n), np.random.uniform(1, np.pi, n), \
                           np.random.uniform(1, 2 * np.pi, n), np.random.uniform(1, 2 * np.pi, n), \
                           np.random.uniform(1, np.pi / 4, 1)

izaDeg, vzaDeg, iaaDeg, vaaDeg, nbarDeg = np.rad2deg(iza), np.rad2deg(vza), np.rad2deg(iaa), np.rad2deg(
    vaa), np.rad2deg(nbar)

raa = iaa - vaa
raaDeg = iaaDeg - vaaDeg

degitems = [izaDeg, vzaDeg, iaaDeg, vaaDeg, raaDeg]
raditems = [iza, vza, iaa, vaa, raa]

degitems_key = ["izaDeg", "vzaDeg", "iaaDeg", "vaaDeg", "raaDeg"]
raditems_key = ["iza", "vza", "iaa", "vaa", "raa"]

property_list = ["len", "shape", "B", "BDeg", "mui", "muv", "phi", "dtype", "nbar", "nbarDeg", "normalize"]

# "geometries", "geometriesDeg", "array", "arraDeg"

property_list_values = [len(iza), (7, len(iza)), (1 / np.cos(iza) + 1 / np.cos(vza)),
                        (1 / np.cos(izaDeg) + 1 / np.cos(vzaDeg)),
                        np.cos(iza), np.cos(vza), np.abs((raa % (2. * np.pi))), random_dtype, nbar, nbarDeg, False]

arrayarray = np.array([iza, vza, raa, iaa, vaa])
arrayDeg = np.array([izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg])

freq = np.random.uniform(0.5, 100000000, n)  # Frequencies in Hz
lam = np.random.uniform(0.01, 1000000, n)  # Frequencies in cm


class TestAlignWithN2:
    def test_align_with_DEG(self):
        align = True
        angle_unit = 'DEG'

        ang = Sensor(value=freq, iza=izaDeg, vza=vzaDeg, iaa=iaaDeg, vaa=vaaDeg,
                     normalize=False, nbar=nbarDeg, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array = ang.align_with(random_array)

        assert len(aligned_random_array[0]) == n2

        for i, item in enumerate(property_list_values):
            try:
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    assert np.allclose(ang[property_list[i]][0:-(n2 - n)], item)

            except (ValueError, TypeError):
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    try:
                        assert ang[property_list[i]] == item
                    except ValueError:
                        print(ang[property_list[i]])
                        print(item)

        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])

    def test_align_with_RAD(self):
        align = True
        angle_unit = 'RAD'

        ang = Sensor(value=freq, iza=iza, vza=vza, iaa=iaa, vaa=vaa,
                     normalize=False, nbar=nbar, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array = ang.align_with(random_array)

        assert len(aligned_random_array[0]) == n2

        # for i, item in enumerate(degitems):  # This Fails sometimes because of the random numbers.
        #     assert np.allclose(ang[degitems_key[i]][0:-(n2 - n)], item)

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]][n:-1], item[-1])

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]][0:-(n2 - n)], item)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]][n:-1], item[-1])

        for i, item in enumerate(property_list_values):
            try:
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    assert np.allclose(ang[property_list[i]][0:-(n2 - n)], item)

            except (ValueError, TypeError):
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    try:
                        assert ang[property_list[i]] == item
                    except ValueError:
                        print(ang[property_list[i]])
                        print(item)

        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])


class TestAlignWithN3:
    def test_align_with_DEG(self):
        align = True
        angle_unit = 'DEG'

        ang = Sensor(value=freq, iza=izaDeg, vza=vzaDeg, iaa=iaaDeg, vaa=vaaDeg,
                     normalize=False, nbar=nbarDeg, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array = ang.align_with(random_array2)

        assert len(aligned_random_array[0]) == n

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]], item)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]], item)

        for i, item in enumerate(property_list_values):
            try:
                assert np.allclose(ang[property_list[i]], item)
            except (ValueError, TypeError):
                assert ang[property_list[i]] == item

        
        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])

    def test_align_with_RAD(self):
        align = True
        angle_unit = 'RAD'

        ang = Sensor(value=freq, iza=iza, vza=vza, iaa=iaa, vaa=vaa,
                     normalize=False, nbar=nbar, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array = ang.align_with(random_array2)

        assert len(aligned_random_array[0]) == n

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]], item)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]], item)

        for i, item in enumerate(property_list_values):
            try:
                assert np.allclose(ang[property_list[i]], item)
            except (ValueError, TypeError):
                try:
                    assert ang[property_list[i]] == item
                except ValueError:
                    print(ang[property_list[i]])
                    print(item)

        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])

    def test_align_with_RAD_NORMALIZE(self):
        angle_unit = 'RAD'

        ang = Sensor(value=freq, iza=iza, vza=vza, iaa=iaa, vaa=vaa,
                     normalize=True, nbar=nbar, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array = ang.align_with(random_array2)

        assert len(aligned_random_array[0]) == n + 1

        assert ang.len == n + 1

        for i, item in enumerate(degitems):
            if 'iza' in degitems_key[i]:
                assert np.allclose(ang[degitems_key[i]][-1], nbarDeg)
            else:
                assert np.allclose(ang[degitems_key[i]][-1], 0)

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]][0:-1], item)

        for i, item in enumerate(raditems):
            if 'iza' in raditems_key[i]:
                assert np.allclose(ang[raditems_key[i]][-1], nbar)
            else:
                assert np.allclose(ang[raditems_key[i]][-1], 0)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]][0:-1], item)


class TestAlignWithNN23:
    def test_align_with_DEG(self):
        align = True
        angle_unit = 'DEG'

        ang = Sensor(value=freq, iza=izaDeg, vza=vzaDeg, iaa=iaaDeg, vaa=vaaDeg,
                     normalize=False, nbar=nbarDeg, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array, aligned_random_array2 = ang.align_with((random_array, random_array2))

        assert len(aligned_random_array) == n2
        assert len(aligned_random_array2) == n2

        # for i, item in enumerate(degitems):  # This Fails sometimes because of the random numbers.
        #     assert np.allclose(ang[degitems_key[i]][0:-(n2 - n)], item)

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]][n:-1], item[-1])

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]][0:-(n2 - n)], item)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]][n:-1], item[-1])

        for i, item in enumerate(property_list_values):
            try:
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    assert np.allclose(ang[property_list[i]][0:-(n2 - n)], item)
            except (ValueError, TypeError):
                if property_list[i] == 'len':
                    assert ang[property_list[i]] == n2
                elif property_list[i] == 'shape':
                    assert ang[property_list[i]] == (7, n2)
                else:
                    assert ang[property_list[i]] == item

        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])

    def test_align_with_DEGN3(self):
        align = True
        angle_unit = 'DEG'

        ang = Sensor(value=freq, iza=izaDeg, vza=vzaDeg, iaa=iaaDeg, vaa=vaaDeg,
                     normalize=False, nbar=nbarDeg, angle_unit=angle_unit, dtype=random_dtype)

        aligned_random_array, aligned_random_array2 = ang.align_with((random_array2, random_array2))

        assert len(aligned_random_array) == n
        assert len(aligned_random_array2) == n

        for i, item in enumerate(degitems):
            assert np.allclose(ang[degitems_key[i]], item)

        for i, item in enumerate(raditems):
            assert np.allclose(ang[raditems_key[i]], item)

        for i, item in enumerate(property_list_values):
            try:
                assert np.allclose(ang[property_list[i]], item)
            except (ValueError, TypeError):
                assert ang[property_list[i]] == item

        assert ang.align == align
        assert ang.angle_unit == angle_unit

        raditems_geom = [iza, vza, raa, iaa, vaa]
        degitems_geom = [izaDeg, vzaDeg, raaDeg, iaaDeg, vaaDeg]

        for i in range(len(iza)):
            radgeom = ang.geometries[i]
            degdgeom = ang.geometriesDeg[i]
            ind = radgeom[0:-2]
            for j, _ in enumerate(ind):
                assert np.allclose(radgeom[j], raditems_geom[j][i])
                assert np.allclose(degdgeom[j], degitems_geom[j][i])



# -*- coding: utf-8 -*-
"""
Created on 07.04.19 by ibaris
"""
from __future__ import division

import numpy as np
from scipy.stats import pearsonr

from rspy.specials.stats import *

n = 50

x = np.ones(10)
y = np.ones(10)


class TestCorr:
    def test_pearson(self):
        for kaka in range(n):
            x = np.random.randn(50)
            y = np.random.randn(50)

            ps = pearsonr(x, y)[0]
            pss = ps ** 2
            pr = pearson(x, y)
            prs = pearson(x, y, True)

            assert np.allclose(ps, pr)
            assert np.allclose(pss, prs)

    def test_rmse(self):
        assert rmse(x, y) == 0
        assert rmsd(x, y) == 0
        assert urmsd(x, y) == 0
        assert urmse(x, y) == 0



# -*- coding: utf-8 -*-
"""
Created on 01.04.2019 by Ismail Baris
"""
import sympy as sym

__all__ = ['Frequency', 'Length', 'Energy', 'Power', 'Time', 'Temperature', 'Mass', 'Current', 'Other',
           'Volume', 'Area', 'Angle', 'Zero', 'One', 'zoo', 'oo', 'NaN']

__NONE_TYPE_UNITS__ = [sym.S.Zero, None, sym.zoo, sym.oo, sym.S.NaN]
__NONE_TYPE_DIMS__ = [None, sym.zoo, sym.oo, sym.S.NaN]
__SYMPY_CLASSES__ = tuple(sym.core.all_classes)

Zero = sym.S.Zero
One = sym.S.One
zoo = sym.zoo
oo = sym.oo
NaN = sym.S.NaN


class Frequency(dict):
    """ Storage for frequency units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Length(dict):
    """ Storage for length units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Energy(dict):
    """ Storage for energy units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Power(dict):
    """ Storage for power units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Time(dict):
    """ Storage for time units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Temperature(dict):
    """ Storage for temperature units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Mass(dict):
    """ Storage for mass units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Current(dict):
    """ Storage for current units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Other(dict):
    """ Storage for other, dimensionless units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Volume(dict):
    """ Storage for Volume units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Area(dict):
    """ Storage for Volume units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


class Angle(dict):
    """ Storage for other, angle units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())



# -*- coding: utf-8 -*-
"""
Created on 01.04.2019 by Ismail Baris
"""
from __future__ import division

import numpy as np

from rspy import Units
from rspy.units.auxiliary import __NONE_TYPE_UNITS__, Zero
from rspy.units.utility import *

n = 10


class TestNoneUnitsAndDimensions:
    def test_none_unit(self):
        units = Units.units.values()

        for item in units:
            assert (not unit_isnone(item))

        for x in range(n):
            assert (unit_isnone(np.random.choice(__NONE_TYPE_UNITS__)))

    def test_none_dim(self):
        units = [Units.angle.values(), Units.area.values(), Units.current.values(), Units.energy.values(),
                 Units.frequency.values(), Units.length.values(), Units.mass.values(), Units.power.values(),
                 Units.temperature.values(), Units.time.values()]

        for i, _ in enumerate(units):
            items = units[i]

            for item in items:
                assert (not dim_isnone(item))

        for x in range(n):
            assert (dim_isnone(np.random.choice(__NONE_TYPE_UNITS__)))

    def test_one_dim(self):
        units = Units.other.values()

        for item in units:
            assert dim_isone(item)

        for x in range(n):
            assert (not dim_isone(np.random.choice(__NONE_TYPE_UNITS__)))

    def test_zero_dim(self):
        assert not dim_iszero(Zero)



# -*- coding: utf-8 -*-
"""
Created on 31.12.2019 by Ismail Baris
"""
from __future__ import division

import warnings

import numpy as np

from rspy.ancillary import *
from rspy.auxiliary.types import __DTYPES__

warnings.filterwarnings('ignore')

n = np.random.randint(8, 12)
nmax = n + 2

random_dtypes = np.random.choice(__DTYPES__, n)
random_dtype = np.random.choice(__DTYPES__, 1)[0]

max_length_array = np.random.random(nmax).astype(random_dtype)

random_arrays = list()
for i in range(n):
    array = np.random.random(np.random.randint(1, n))
    random_arrays.append(array.astype(random_dtypes[i]))

random_arrays.append(max_length_array)


class TestAlignment:
    def test_align_all_dtype_none(self):
        assert not same_len(random_arrays)
        assert not same_len(3)
        assert not same_shape(random_arrays)

        aligned = align_all(tuple(random_arrays))

        for i in range(len(aligned) - 1):
            assert len(aligned[i]) == nmax
            assert aligned[i].dtype == random_dtypes[i]

        assert same_len(aligned)
        assert same_shape(aligned)

    def test_align_all_dtype_random(self):
        assert not same_len(random_arrays)
        assert not same_shape(random_arrays)

        aligned = align_all(tuple(random_arrays), dtype=random_dtype)

        for i in range(len(aligned) - 1):
            assert len(aligned[i]) == nmax
            assert aligned[i].dtype == random_dtype

        assert same_len(aligned)
        assert same_shape(aligned)

    def test_align_all_wc_dtype_none(self):
        assert not same_len(random_arrays)
        assert not same_shape(random_arrays)

        c_value = -9
        aligned = align_all(tuple(random_arrays), constant_values=c_value)

        for i in range(len(aligned) - 1):
            assert len(aligned[i]) == nmax
            assert aligned[i][-1] == random_dtypes[i](c_value)
            assert aligned[i].dtype == random_dtypes[i]

        assert same_len(aligned)
        assert same_shape(aligned)

    def test_align_all_wc_dtype_random(self):
        assert not same_len(random_arrays)
        assert not same_shape(random_arrays)

        c_value = -9
        aligned = align_all(tuple(random_arrays), dtype=random_dtype, constant_values=c_value)

        for i in range(len(aligned) - 1):
            assert len(aligned[i]) == nmax
            assert aligned[i][-1] == random_dtypes[i](c_value)
            assert aligned[i].dtype == random_dtype

        assert same_len(aligned)
        assert same_shape(aligned)


class TestOther:
    def test_get_dtypes(self):
        dtypes = get_dtypes(tuple(random_arrays))

        for i in range(len(dtypes) - 1):
            assert dtypes[i] == random_dtypes[i]

    def test_max_length(self):
        assert max_length(tuple(random_arrays)) == nmax

    def test_asarrays(self):
        lists = [2, 3, 4, 5, [1, 2, 3], 456]
        arrays = asarrays(lists)

        for item in arrays:
            assert isinstance(item, np.ndarray)

    def test_same_len_shape_error(self):
        assert not same_len(('aa', 'bb', 'pipi', 'kaka', np.dtype))
        assert not same_shape(('aa', 'bb', 'pipi', 'kaka', np.dtype))

    def test_zeros_likes(self):
        rand_int = np.random.randint(1, 10)

        data = zeros_likes(random_arrays[-1], rep=rand_int)

        for i, item in enumerate(data):
            assert item.shape == random_arrays[-1].shape
            assert item.dtype == random_arrays[-1].dtype

        data = zeros_likes(random_arrays[-1], rep=rand_int, dtype=random_dtype)

        for i, item in enumerate(data):
            assert item.shape == random_arrays[-1].shape
            assert item.dtype == random_dtype



# -*- coding: utf-8 -*-
"""
Created on  by Ismail Baris
"""
from __future__ import division
import numpy as np

__all__ = ['pi', 'c', 'optical_range']

pi = 3.1415926535897932384626433832795028841971
c = 299792458.0  # Speed of light in m / s
optical_range = np.arange(400, 2501, 1)  # Wavelength in optical range in nm



# -*- coding: utf-8 -*-
"""
Created on 03.04.19 by ibaris
"""
from numpy import add, subtract, multiply, true_divide, power

__all__ = ['Operator']


class Operator(dict):
    """ Storage for frequency units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    __operators = (add, subtract, multiply, true_divide, power)
    __sym_keys = ('+', '-', '*', '/', '**')
    __class_keys = ('__add__', '__sub__', '__mul__', '__truediv__', '__pow__')
    __numpy_keys = ('add', 'subtract', 'multiply', 'true_divide', 'power')

    def __init__(self):
        self.operators = dict(zip(Operator.__sym_keys, Operator.__operators))
        self.operators.update(dict(zip(Operator.__class_keys, Operator.__operators)))
        self.operators.update(dict(zip(Operator.__numpy_keys, Operator.__operators)))

        for item in self.operators.keys():
            self[item] = self.operators[item]

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())


Operator = Operator()



# -*- coding: utf-8 -*-
"""
Created on 06.04.19 by ibaris
"""
from __future__ import division

import numpy as np
import pytest

from rspy import Waves, Units, UnitError

n = 50

n1 = np.random.randint(60, 101)

freq = np.random.uniform(0.5, 100000000, n1)  # Frequencies in Hz
lam = np.random.uniform(0.01, 1000000, n1)  # Frequencies in cm


class TestInit:
    def test_freq(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(freq, unit1, unit2)

            assert np.allclose(w.frequency, freq)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2

    def test_lam(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(lam, unit2, unit1)

            assert np.allclose(w.wavelength, lam)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2

    def test_utility(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(freq, unit1, unit2)

            assert len(w) == len(freq)
            assert w.len == len(freq)
            assert w.shape == freq.shape

    def test_excep(self):
        with pytest.raises(UnitError):
            w = Waves(lam, 'cm', 'cm')

        with pytest.raises(UnitError):
            w = Waves(freq, 'Ghz', 'GHz')



# -*- coding: utf-8 -*-
"""
Created on 31.03.2019 by Ismail Baris

For COPYING and LICENSE details, please refer to the LICENSE file
(c) 2017- Ismail Baris

"""
import os
from glob import glob

import numpy
from setuptools import find_packages

try:
    from Cython.Distutils import build_ext

except ImportError:
    USING_CYTHON = False

else:
    USING_CYTHON = True

try:
    from setuptools import setup
    from setuptools import Extension
except ImportError:
    from distutils.core import setup
    from distutils.extension import Extension


# ------------------------------------------------------------------------------------------------------------
# Environmental Functions
# ------------------------------------------------------------------------------------------------------------
def get_version():
    """
    Function to get the version.

    Returns
    -------
    out : str
    """
    version = dict()

    with open("rspy/__version__.py") as fp:
        exec (fp.read(), version)

    return version['__version__']


def get_packages():
    """
    Function get necessary modules.

    Returns
    -------
    out : setuptools.find_packages
    """
    return find_packages(exclude=['docs'])


# Open and Read Requirements --------------------------------------------------------------------------------
with open('requirements.txt') as f:
    required = f.read().splitlines()

# ------------------------------------------------------------------------------------------------------------
# Build Cython Extensions
# ------------------------------------------------------------------------------------------------------------
ext_modules = []
cmdclass = {}

ext = 'pyx' if USING_CYTHON else 'c'  # Define extension. If cython is present then use the pyx files.
sources = glob('rspy/**/*/*.{0}'.format(ext))  # Define source directories.

for item in sources:
    ext_modules.append(Extension(item.split('.')[0].replace(os.path.sep, '.'),
                                 [item],
                                 include_dirs=['rspy/bin', os.path.dirname(item), '.']))

if USING_CYTHON:
    cmdclass.update({'build_ext': build_ext})

setup(name='rspy',
      author="Ismail Baris",
      maintainer='Ismail Baris',
      version=get_version(),
      description='Fundamental Formulas for Radar and Angle Management',

      # Build Extensions -----------------------------------------------------------------------------------
      packages=get_packages(),
      cmdclass=cmdclass,
      include_dirs=[numpy.get_include()],
      ext_modules=ext_modules,

      # General Package Information ------------------------------------------------------------------------
      # ~ license='APACHE 2',
      url='https://github.com/ibaris/rspy',
      long_description='Fundamental Formulas for Radar and Angle Management',

      keywords=["radar", "remote-sensing", "optics", "integration",
                "microwave", "estimation", "physics", "radiative transfer"],

      # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Intended Audience :: Science/Research',
          'Topic :: Scientific/Engineering :: Atmospheric Science',
          "Development Status :: 5 - Production/Stable",
          "Intended Audience :: Developers",
          "Natural Language :: English",
          "License :: Other/Proprietary License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4"],

      # Include Package Data -------------------------------------------------------------------------------
      package_data={'rspy/bin/bin_unit': ['*.pxd', '*.c']},
      include_package_data=True,
      zip_safe=False,

      # Package Requirement --------------------------------------------------------------------------------
      install_requires=required,

      # Package Testing ------------------------------------------------------------------------------------
      setup_requires=['pytest-runner'],
      tests_require=['pytest'])



# -*- coding: utf-8 -*-
"""
Created on 04.04.19 by ibaris
"""
from __future__ import division

import numpy as np
from sympy.physics.units import convert_to as sympy_convert_to

from rspy import Units

n = 10

dimensions = ['frequency',  # Other Dimensions have one entries
              'length',
              'mass',
              'time']


class TestBothDimensions:
    def test_single_dimensions(self):

        values = np.random.uniform(0.1, 500, 100)

        for x in range(n):
            for i, item in enumerate(dimensions):
                j = np.random.randint(0, 100)
                units = Units[item]

                ind = np.random.choice(len(units.values()))
                unit_values = list(units.values())
                unit1 = unit_values[ind]
                unit2 = unit1

                while unit2 == unit1:
                    ind = np.random.choice(len(units.values()))
                    unit_values = list(units.values())
                    unit2 = unit_values[ind]

                sympy_conversion = sympy_convert_to(unit1 * values[j], unit2)
                sympy_result = round(float(sympy_conversion.args[0]), 4)

                rspy_conversion = Units.convert_to(values[j], unit1, unit2).round(4)

                assert np.allclose(sympy_result, rspy_conversion)

    def test_inverse_dimensions_length_to_time(self):

        values = np.random.uniform(0.1, 500, 100)

        for x in range(n):
            j = np.random.randint(0, 100)
            units_length = Units['length']
            units_time = Units['time']

            # Determine Units of Length
            ind = np.random.choice(len(units_length.values()))
            unit_values = list(units_length.values())
            unit1_l = unit_values[ind]

            # Determine Units of Time
            ind = np.random.choice(len(units_time.values()))
            unit_values = list(units_time.values())
            unit1_s = unit_values[ind]

            unit1 = unit1_l / unit1_s
            unit2 = unit1

            while unit2 == unit1:
                # Determine Units of Length
                ind = np.random.choice(len(units_length.values()))
                unit_values = list(units_length.values())
                unit2_l = unit_values[ind]

                # Determine Units of Time
                ind = np.random.choice(len(units_time.values()))
                unit_values = list(units_time.values())
                unit2_s = unit_values[ind]

                unit2 = unit2_l / unit2_s

            sympy_conversion = sympy_convert_to(unit1 * values[j], unit2)
            sympy_result = round(float(sympy_conversion.args[0]), 4)

            rspy_conversion = Units.convert_to(values[j], unit1, unit2).round(4)

            assert np.allclose(sympy_result, rspy_conversion)

    def test_inverse_dimensions(self):

        values = np.random.uniform(0.1, 500, 100)

        for x in range(n):
            for i, item in enumerate(dimensions):
                j = np.random.randint(0, 100)
                units = Units[item]

                ind = np.random.choice(len(units.values()))
                unit_values = list(units.values())
                unit1 = 1 / unit_values[ind]
                unit2 = unit1

                while unit2 == unit1:
                    ind = np.random.choice(len(units.values()))
                    unit_values = list(units.values())
                    unit2 = 1 / unit_values[ind]

                sympy_conversion = sympy_convert_to(unit1 * values[j], unit2)
                sympy_result = round(float(sympy_conversion.args[0]), 4)

                rspy_conversion = Units.convert_to(values[j], unit1, unit2).round(4)

                assert np.allclose(sympy_result, rspy_conversion)


class TestCycle:
    def test_hz_cycle_unit1(self):
        values = np.random.uniform(0.1, 500, 100)

        for x in range(n):
            j = np.random.randint(0, 100)
            units = Units['frequency']

            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = 1 / Units.time.s
            unit2 = unit_values[ind]

            sympy_conversion = sympy_convert_to(unit1 * values[j], unit2)
            sympy_result = round(float(sympy_conversion.args[0]), 4)

            rspy_conversion = Units.convert_to(values[j], unit1, unit2).round(4)

            assert np.allclose(sympy_result, rspy_conversion)

    def test_hz_cycle_unit2(self):
        values = np.random.uniform(0.1, 500, 100)

        for x in range(n):
            j = np.random.randint(0, 100)
            units = Units['frequency']

            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]
            unit2 = 1 / Units.time.s

            sympy_conversion = sympy_convert_to(unit1 * values[j], unit2)
            sympy_result = round(float(sympy_conversion.args[0]), 4)

            rspy_conversion = Units.convert_to(values[j], unit1, unit2).round(4)

            assert np.allclose(sympy_result, rspy_conversion)


class TestTempreture:
    def from_kelvin(self):
        reference_to_celsius = {300: 26.85,
                                125: -148.15,
                                671: 397.85,
                                314: 40.85,
                                436: 162.85}

        reference_to_fahrenheit = {300: 80.33,
                                   125: -234.67,
                                   671: 748.13,
                                   314: 105.53,
                                   436: 325.13}

        for item in reference_to_celsius.keys():
            assert np.allclose(reference_to_celsius[item], Units.convert_to(item, 'K', 'celsius'))

        for item in reference_to_fahrenheit.keys():
            assert np.allclose(reference_to_fahrenheit[item], Units.convert_to(item, 'K', 'fahrenheit'))

    def from_celsius(self):
        reference_to_kelvin = {26.85: 300,
                               -148.15: 125,
                               397.85: 671,
                               40.85: 314,
                               162.85: 436}

        reference_to_fahrenheit = {26.85: 80.33,
                                   -148.15: -234.67,
                                   397.85: 748.13,
                                   40.85: 105.53,
                                   162.85: 325.13}

        for item in reference_to_kelvin.keys():
            assert np.allclose(reference_to_kelvin[item], Units.convert_to(item, 'celsius', 'K'))

        for item in reference_to_fahrenheit.keys():
            assert np.allclose(reference_to_fahrenheit[item], Units.convert_to(item, 'celsius', 'fahrenheit'))

    def from_fahrenheit(self):

        reference_to_celsius = {80.33: 26.85,
                                -234.67: -148.15,
                                748.13: 397.85,
                                105.53: 40.85,
                                325.13: 162.85}

        reference_to_kelvin = {80.33: 300,
                               -234.67: 125,
                               748.13: 671,
                               105.53: 314,
                               325.13: 436}

        for item in reference_to_celsius.keys():
            assert np.allclose(reference_to_celsius[item], Units.convert_to(item, 'fahrenheit', 'celsius'))

        for item in reference_to_kelvin.keys():
            assert np.allclose(reference_to_kelvin[item], Units.convert_to(item, 'fahrenheit', 'K'))



# -*- coding: utf-8 -*-
"""
Created on 06.04.19 by ibaris
"""
from __future__ import division

import numpy as np

import rspy.constants as const
from rspy import Waves, Units

n = 50

n1 = np.random.randint(60, 101)
n2 = np.random.randint(120, 150)
n3 = np.random.randint(30, 50)

freq = np.random.uniform(0.5, 100000000, n1)  # Frequencies in Hz
lam = np.random.uniform(0.01, 1000000, n1)  # Frequencies in cm

random_array = np.random.uniform(1, np.pi / 2, n2)
random_array2 = np.random.uniform(1, np.pi / 2, n3)


class TestConversion:
    def test_freq(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(freq, unit1, unit2)

            frequency = Units.convert_to(freq, unit1, '1 / s')

            lam_t = const.c / frequency
            lam_c = Units.convert_to(lam_t, 'm', unit2)
            wnb = 2 * const.pi / lam_c

            assert np.allclose(w.frequency, freq)
            assert np.allclose(w.wavelength, lam_c)
            assert np.allclose(w.wavenumber, wnb)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2
            assert len(w) == len(freq)
            assert w.len == len(freq)
            assert w.shape == freq.shape

    def test_lamd(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(lam, unit2, unit1)
            wnb = 2 * const.pi / lam

            c = Units.convert_to(const.c, 'm / s', unit2 / Units.time.s)
            f = c / lam

            f_con = Units.convert_to(f, 'Hz', unit1)

            assert np.allclose(w.frequency, f_con)
            assert np.allclose(w.wavelength, lam)
            assert np.allclose(w.wavenumber, wnb)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2


class TestAlign:
    def test_align(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(freq, unit1, unit2)

            aligned_random_array = w.align_with(random_array)

            assert len(aligned_random_array[0]) == n2

            assert np.allclose(w.frequency[0:-(n2 - n1)], freq)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2

    def test_align2(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(freq, unit1, unit2)

            aligned_random_array = w.align_with(random_array2)

            assert len(aligned_random_array[0]) == n1

            assert np.allclose(w.frequency, freq)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2

    def test_align3(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(lam, unit2, unit1)

            aligned_random_array = w.align_with(random_array)

            assert len(aligned_random_array[0]) == n2

            assert np.allclose(w.wavelength[0:-(n2 - n1)], lam)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2

    def test_align4(self):
        for x in range(n):
            units = Units['frequency']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit1 = unit_values[ind]

            units = Units['length']
            ind = np.random.choice(len(units.values()))
            unit_values = list(units.values())
            unit2 = unit_values[ind]

            w = Waves(lam, unit2, unit1)

            aligned_random_array = w.align_with(random_array2)

            assert len(aligned_random_array[0]) == n1

            assert np.allclose(w.wavelength, lam)
            assert w.frequency_unit == unit1
            assert w.wavelength_unit == unit2
            assert w.wavenumber_unit == 1 / unit2



# -*- coding: utf-8 -*-
"""
Created on 31.03.2019 by Ismail Baris
"""
from __future__ import division

import numpy as np

__all__ = ['valid_dtype', 'valid_angle_deg', 'valid_angle_rad', 'check_angle_unit']

__DTYPES__ = [np.float, np.float16,
              np.double, np.longdouble,
              np.float32, np.float64,
              float]

__UNIT_RAD__ = ['RAD', 'rad', 'radian', 'radians']
__UNIT_DEG__ = ['DEG', 'deg', 'degree', 'degrees']


def valid_angle_deg(unit):
    if unit in __UNIT_DEG__:
        return True

    return False


def valid_angle_rad(unit):
    if unit in __UNIT_RAD__:
        return True

    return False


def check_angle_unit(unit):
    if valid_angle_deg(unit) or valid_angle_rad(unit):
        return True
    else:
        raise ValueError("Unit `{0}` not understood. Supported angle units "
                         "are: {1} and {2}".format(str(unit), __UNIT_RAD__, __UNIT_DEG__))


def valid_dtype(dtype):
    """
    Check whether a dtype is supported or not.

    Parameters
    ----------
    dtype : object
        A dtype object.

    Returns
    -------
    bool
    """
    if dtype in __DTYPES__:
        return True

    return False



# -*- coding: utf-8 -*-
# cython: cdivision=True
"""
Created on 03.04.19 by ibaris
"""
from __future__ import division
from rspy.bin.bin_units.conversion cimport bin_sym_convert_to
from rspy.bin.bin_units.dtypes cimport DTYPE_ARRAY

def sym_convert_to(DTYPE_ARRAY expr, object unit):
    return bin_sym_convert_to(expr, unit)



# -*- coding: utf-8 -*-
"""
Created on 01.04.2019 by Ismail Baris
"""
from __future__ import division

import numpy as np

from rspy.auxiliary import Operator, UnitError
from rspy.bin.bin_units import sym_convert_to
from rspy.units.auxiliary import (Frequency, Length, Energy, Power, Time, Temperature, Mass, Current, Other,
                                  Volume, Area, Angle, Zero)
from rspy.units.dimensions import (frequency, length, energy, power, time, temperature, mass,
                                   current, area, volume, angle)
from rspy.units.si_units import __unit__, __values__
from rspy.units.utility import (unit_isnone, dim_isnone, dim_isone, dim_iszero, isexpr)

__all__ = ['Units']


class Units(dict):
    """ Storage for all units.

    Returns
    -------
    Dict with .dot access.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method. adar Backscatter values of multi scattering contribution of surface and volume
    """

    __Frequency = Frequency()
    __Length = Length()
    __Energy = Energy()
    __Power = Power()
    __Time = Time()
    __Temperature = Temperature()
    __Mass = Mass()
    __Current = Current()
    __Other = Other()
    __Area = Area()
    __Angle = Angle()
    __Volume = Volume()

    __unit_dict__ = {"frequency": __Frequency,
                     'length': __Length,
                     "energy": __Energy,
                     "power": __Power,
                     "time": __Time,
                     "temperature": __Temperature,
                     'mass': __Mass,
                     "current": __Current,
                     'other': __Other,
                     'area': __Area,
                     'volume': __Volume,
                     'angle': __Angle}

    dimensions = {'angle': angle, 'area': area, 'volume': volume, 'frequency': frequency, 'length': length,
                  'energy': energy, 'power': power,
                  'temperature': temperature, 'time': time, 'mass': mass,
                  'current': current}

    def __init__(self):
        self.__unit_dict = Units.__unit_dict__
        self.units = dict(zip(__unit__, __values__))
        self.dimensions = Units.dimensions

        self.__setup_units()

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("{} is not a valid unit. Use `keys()` method to see all available units".format(name))

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])

        return self.__class__.__name__ + "()"

    def __dir__(self):
        return list(self.keys())

    def __setup_units(self):
        """
        Setup Unit class with all available units and dimensions.

        Returns
        -------
        None
        """
        for item in self.units.keys():
            dimension = self.units[item].dimension.name

            if dim_isnone(self.units[item]) or dim_isone(self.units[item]) or dim_iszero(self.units[item]):
                self.__unit_dict['other'][str(item)] = self.units[item]
            else:
                self.__unit_dict[str(dimension)][str(item)] = self.units[item]

        for item in self.__unit_dict.keys():
            self[item] = self.__unit_dict[item]

    @staticmethod
    def unit_isnone(unit):
        """
        Check if a unit has a None-Typed object.

        Parameters
        ----------
        unit : object
            Unit expression.

        Returns
        -------
        bool
        """
        return unit_isnone(unit)

    @staticmethod
    def dim_isnone(unit):
        """
        Check if a dimension of a unit has a None-Typed object.

        Parameters
        ----------
        unit : object
            Unit expression.

        Returns
        -------
        bool
        """
        return dim_isnone(unit)

    @staticmethod
    def dim_isone(unit):
        """
        Check if a dimension of a unit is One.

        Parameters
        ----------
        unit : object
            Unit expression.

        Returns
        -------
        bool
        """
        return dim_isone(unit)

    @staticmethod
    def dim_iszero(unit):
        """
        Check if a dimension of a unit is Zero.

        Parameters
        ----------
        unit : object
            Unit expression.

        Returns
        -------
        bool
        """
        return dim_iszero(unit)

    @staticmethod
    def isexpr(value):
        """
        Check if a object is sympy expression.

        Parameters
        ----------
        value : object or numpy.ndarray
            Expression.

        Returns
        -------
        bool
        """
        return isexpr(value)

    @staticmethod
    def str2unit(unit):
        """
        Get a unit object from str or char.

        Parameters
        ----------
        unit : char*
            Desired unit in str format.

        Returns
        -------
        object

        """

        unit_str = unit.split()
        unit_list = []
        operand_list = []

        for item in unit_str:
            if item in Operator.keys():
                operand_list.append(item)

            else:
                try:
                    item = int(item)
                    unit_list.append(item)

                except ValueError:
                    try:
                        unit_list.append(Units.units[item])

                    except KeyError:
                        raise UnitError(
                            "{} is not a valid unit. There should be spaces between the characters.".format(str(item)))

        unit_obj = unit_list[0]

        for i, item in enumerate(unit_list, 1):
            try:
                item = unit_list[i]
                unit_obj = Operator[operand_list[i - 1]](unit_obj, item)
            except IndexError:
                pass

        return unit_obj

    @staticmethod
    def unit2str(unit):
        if Units.isexpr(unit):
            return '[' + str(unit) + ']'

        return '[' + '-' + ']'

    @staticmethod
    def get_unit(unit):
        """
        Get unit object from string or unit object (sympy).

        Parameters
        ----------
        unit : str, char or unit object.

        Returns
        -------
        object
        """
        if isinstance(unit, str):
            return Units.str2unit(unit)
        elif Units.unit_isnone(unit):
            return Zero
        elif Units.isexpr(unit):
            return unit
        else:
            raise UnitError("{} is not a valid unit.".format(str(unit)))

    @staticmethod
    def convert_to(value, unit, to_unit):

        if isinstance(unit, str):
            unit = Units.str2unit(unit)

        if isinstance(to_unit, str):
            to_unit = Units.str2unit(to_unit)

        if hasattr(unit, 'dimension') and not hasattr(to_unit, 'dimension'):
            if to_unit == 1 / Units.time.s and unit.dimension == Units.dimensions['frequency']:
                scaled_value = value * unit.scale_factor
                value = scaled_value / Units[str(unit.dimension.name)]['Hz'].scale_factor

                return np.atleast_1d(np.asarray(value, dtype=np.double))

        elif hasattr(to_unit, 'dimension') and not hasattr(unit, 'dimension'):
            if unit == 1 / Units.time.s and to_unit.dimension == Units.dimensions['frequency']:
                scaled_value = value

                value = scaled_value / Units[str(to_unit.dimension.name)][str(to_unit)].scale_factor

                return np.atleast_1d(np.asarray(value, dtype=np.double))

        elif hasattr(to_unit, 'dimension') and hasattr(unit, 'dimension'):
            if to_unit.dimension == unit.dimension:

                # Convert tempretures
                if unit.dimension == Units.dimensions['temperature']:

                    # From Celsius to other Units.
                    if unit == Units.temperature.celsius:
                        if to_unit == Units.temperature.kelvin:
                            value = value + unit.scale_factor

                        if to_unit == Units.temperature.fahrenheit:
                            value = value * 9. / 5. + 32

                    # From Kelvin to other Units.
                    if unit == Units.temperature.kelvin:
                        if to_unit == Units.temperature.celsius:
                            value = value - to_unit.scale_factor

                        if to_unit == Units.temperature.fahrenheit:
                            value = value * to_unit.scale_factor - 459.67

                    # From Fahrenheit to other Units.
                    if unit == Units.temperature.fahrenheit:
                        if to_unit == Units.temperature.celsius:
                            value = (value - 32) / unit.scale_factor
                        if to_unit == Units.temperature.kelvin:
                            value = (value + 459.67) * 5./9.

                else:
                    scaled_value = value * Units[str(to_unit.dimension.name)][str(unit)].scale_factor

                    value = scaled_value / Units[str(to_unit.dimension.name)][str(to_unit)].scale_factor

                return np.atleast_1d(np.asarray(value, dtype=np.double))

        if isinstance(value, np.ndarray):
            shape = value.shape
            expr = value.astype(np.double).flatten() * unit

            conversion = sym_convert_to(expr, to_unit)

            return conversion.reshape(shape)

        else:
            value = np.asanyarray(value, dtype=np.double).flatten()
            expr = value * unit

        return sym_convert_to(expr, to_unit).base


Units = Units()
